# Mira Password Manager (Lite)

Mira is our innovative password management solution designed specifically for the command-line interface (CLI). With a streamlined and efficient approach, Mira provides a robust solution to the vulnerabilities associated with password management in the digital era. 
                                                                                     
## Objectives

- **Offline Security**: Mira prioritizes offline functionality to ensure password management without dependence on internet connectivity. This approach enhances security by minimizing the attack surface and reducing exposure to online threats.                     
- **Local Storage**: The password manager facilitates secure storage and retrieval of passwords locally on the user's machine, ensuring data confidentiality and reducing reliance on external servers or cloud services.

- **No Cloud Dependencies**: Mira does not rely on external servers or cloud services for enhanced privacy and control. This approach eliminates the risk of unauthorized access or data breaches associated with online storage.

- **Data Confidentiality**: To ensure the privacy of user data, Mira implements encryption and secure storage procedures, employing Argon2 for hashing the user data. This ensures that even if the device is compromised, the stored passwords remain unreadable without the decryption key.

- **Isolated Environment**: Mira operates within the user's system, minimizing exposure to external threats or vulnerabilities. This isolated environment enhances security by reducing the likelihood of unauthorized access or data breaches.

- **Usability**: Mira provides a user-friendly command-line interface for easy interaction, allowing users to efficiently manage their passwords without the need for complex technical knowledge.

- **Security Best Practices**: The password manager encourages secure practices such as safeguarding master keys, enabling Two-Factor Authentication (2FA), and emphasizing password complexity to enhance overall security.

- **Control**: Mira empowers users with complete control over their password manager, as it operates locally within the system and functions entirely offline. This provides users with a sense of independence and control over their sensitive information.

## Why MIRA is Considered Secure?

- **Limited Attack Surface**: Operating offline reduces the potential attack surface, minimizing vulnerabilities and enhancing overall security.

- **No Cloud Dependency**: Mira does not store user data in the cloud, reducing the risk of unauthorized access or data breaches associated with online storage. Users have direct control over their data, enhancing privacy and security.

- **Lower Exposure to Online Threats**: Due to its offline nature, Mira is less susceptible to phishing attacks or server breaches common with online password managers. Offline managers are not reliant on online authentication, reducing exposure to online threats.

- **Local Encryption**: Mira employs strong encryption algorithms to protect user data. Data is stored locally, and encryption ensures that even if the device is compromised, the stored passwords remain encrypted and unreadable without the decryption key.

- **Zero-Knowledge Architecture**: Mira ensures the privacy of user data by handling passwords without actually knowing what they are. It encrypts and decrypts data locally, ensuring sensitive information stays private, even from the password manager itself.

- **User Independence**: Users have full control over their password manager since it operates within their system. There's no reliance on external services, providing a sense of independence and reducing potential points of failure.

- **No Password Retrieval Over Network**: Password retrieval and storage occur locally, eliminating the need to transmit sensitive information over the network. This reduces the chances of interception or man-in-the-middle attacks.

- **Offline Password Generation**: Mira can generate passwords without requiring an internet connection, offering convenience and security, especially in situations where online access is limited.

- **No Third-Party Involvement**: Mira operates autonomously without any reliance on external service providers, ensuring enhanced security by eliminating potential risks associated with third-party entities. Users can fully trust the locally implemented security measures without involving external platforms.

- **Increased Privacy Control**: Users benefit from enhanced privacy control as their sensitive data remains within their own environment. There is no need to trust external servers with personal information, contributing to a more private and secure user experience.

- **No Connection Latency**: Users experience minimal latency since their password data is instantly accessible locally. This absence of network communication reduces the risk of delays or disruptions that might be exploited by attackers attempting to compromise the system.

*While offline password managers provide significant security benefits, users should remain vigilant about security best practices. It's crucial to store the master key and decryption key in a secure location on their device. Unfortunately, human error often poses the primary challenge in adhering to these practices.*

## MIRA's Credential Storage Capabilities

MIRA supports various types of credentials, including:

- **Account Passwords**: MIRA enables users to securely store and manage passwords for a variety of accounts, such as those used for websites, applications, and services. It employs encryption techniques to safeguard these passwords, preventing unauthorized access and ensuring user privacy and security online.

- **Credit/Debit Card PINs**: MIRA provides a secure repository for storing sensitive financial information, including PINs for credit and debit cards. By encrypting this data and implementing robust security measures, MIRA ensures that users can safely store and access their card PINs, protecting them from unauthorized use and fraudulent activities. (PREMIUM)

- **API Keys**: MIRA offers a secure environment for storing API keys, which are authentication credentials used to access web services or APIs. By securely managing these keys, MIRA helps prevent unauthorized access to sensitive data and resources, enhancing the overall security of applications and systems that rely on API authentication. (PREMIUM)

- **SSH(RSA) Keys**: MIRA supports the storage and management of SSH keys, which are cryptographic keys used for secure remote access to servers and systems via the Secure Shell (SSH) protocol. By safeguarding SSH keys within its encrypted storage, MIRA helps users securely authenticate and access remote systems, minimizing the risk of unauthorized access and potential security breaches. (PREMIUM)

- **Source Codes**: MIRA allows users to store and protect their project source codes securely. By providing encrypted storage for source code files, MIRA helps developers safeguard their intellectual property and sensitive project information, ensuring that their code remains confidential and protected from unauthorized access or tampering. (PREMIUM)

- **Private Notes**: MIRA enables users to securely store private information, such as personal notes, sensitive documents, or confidential data. By encrypting these notes and providing secure storage, MIRA helps users safeguard their privacy and sensitive information, ensuring that it remains inaccessible to unauthorized individuals or malicious actors. (PREMIUM)

## Features (Few)

- **Comprehensive Credential Storage**: MIRA offers sophisticated storage capabilities, accommodating an extensive range of credentials. These include Platform Accounts, Credit/Debit Card PINs (exclusive to PREMIUM), API Keys (exclusive to PREMIUM), SSH(RSA) Keys (exclusive to PREMIUM), Source Codes (exclusive to PREMIUM), and Private Notes (exclusive to PREMIUM). This breadth of storage options ensures MIRA's flexibility in securely managing diverse types of credentials for users.

- **Multi-Factor Authentication**: Before the user can go to the main menu of MIRA, the user needs to login first. In login prompt, the user needs to enter the master password and the encryption key that has been generated after successful registration and a time-based 6-digit from 2FA if enabled. (2FA is available in PREMIUM)

- **Secure Storage**: Passwords are encrypted using the Fernet symmetric encryption algorithm, ensuring data confidentiality and integrity. Without the encryption key, it will be almost impossible to retrieve or recover the content.

- **Carrier Lookup**: When you use a phone number for your platform credential instead of email, MIRA will detect its Carrier. This feature can help to validate the phone number that has been entered by the user.

- **Mnemonic Phrase**: Since the encryption key is generated in Base64 format, it's hard to memorize and maintain, that's why MIRA can turn the user's encryption key to a memorable and maintainable mnemonic phrase. Just be careful with hypens and underscores and replace them with a letter, number, or symbols that the user will remember and where they are placed before the user convert it to a mnemonic phrase. (PREMIUM)

- **Password Strength Checker**: Ensures that passwords meet recommended strength criteria based on established password policies. (PREMIUM)

- **Password/PINs Expiry Notifications**: Users receive warnings when passwords are about to expire, prompting them to change passwords regularly for enhanced security. (PREMIUM)

- **Password Generator**: Mira offers various password generation methods to create strong and unique passwords tailored to user preferences. (PREMIUM)

- **Password/PINs Expiry Lister**: Allows users to view the remaining time until a password and PIN expires, facilitating proactive password/PIN management. (PREMIUM)

- **Accessible Instructions**: GiraSec prioritized creating intuitive guidance for MIRA, a Terminal-Based Password Manager, ensuring it offers the utmost user-friendliness.

- **Reset Functionality**: In the Premium Version, MIRA offers a reset option designed to provide users with a comprehensive tool for data management. By selecting this option, users can initiate a process that deletes all saved credentials, including their primary MIRA account. This feature serves as a safeguard, enabling users to reset their MIRA instance in situations such as compromised accounts or a need for a fresh start. (PREMIUM)

*Please note: Resetting MIRA will permanently delete all stored credentials, and this action cannot be undone. Exercise caution and ensure that you have backed up any essential data before proceeding with the reset. MIRA will not be able to recover any deleted credentials after the reset is complete.*

## Installation (PIP)

**UNIX-Based**
```bash
sudo pip3 install MIRA-passwdmng
```
**Windows**:
- Run CMD as Administrator  
```bash
pip3 install MIRA-passwdmng
```

## Usage

To use Mira effectively:

- Run the program with elevated privileges using `sudo mira`.
- Typing `h` or `help` in the Mira prompt provides guidance.
- Before logging in, create a master user to secure the password vault.

Note: For **Windows** users, run the **Command Prompt(CMD)** as **Administrator** to run the program.
  

## Commands

Once logged in, users can utilize various commands, including (FOR LITE ONLY):

| Command               | Description                                                      |
|-----------------------|------------------------------------------------------------------|
| add_platform_passwd   | Add a new password for the desired account ID                    |
| get_platform_passwd   | Retrieve the plaintext version of the password for the account ID |
| del_platform_passwd   | Delete a saved password for the desired account ID               |
| ch_platform_pass      | Change the password for the desired account ID                   |
| list_passwd           | List all the stored credentials on your vault                    |
| lout                  | Logout                                                           |
| exit                  | Terminate MIRA                                                   |

## Security Recommendations

To enhance security:

- Keep master and encryption keys secure to prevent unauthorized access.
- Use complex passwords to safeguard against brute-force attacks and unauthorized access attempts.

## Go Premium for Lifetime Access

Unlock a world of possibilities with our Premium membership:
- **Debit/Credit Card PINs are supported**
- **API Keys are supported**
- **SSH(RSA) Keys are supported**
- **Source Codes are supported**
- **Private Notes are supported**
- **No storage limit for Passwords/PINs/SSH(RSA) Keys/API Keys/Source Codes/Private Notes**
- **2-Factor Authentication**
- **User can change the Master Password anytime**
- **Password Generator**
- **Password Strength Checker**
- **Password/PIN Expiry Checker**
- **Password/PIN Expiry Alerts**
- **Account Lockout**
- **Mnemonic Option for encryption key**
- **Reset Option**

*Upgrade today for lifetime access and experience the robustness of GiraSec's MIRA for only $4.50 (250 PHP)!*

## License

- MIRA is licensed under **EULA**.

## For Premium Access, Concerns, and Issues please email us here:

- girasesolutions@gmail.com
- fredmarkivand@gmail.com
- johnrussel205@gmail.com

## Acknowledgements

- GiraSec Solutions
- veilwr4ith
- icode3rror# Mira Password Manager (Lite)

Mira is our innovative password management solution designed specifically for the command-line interface (CLI). With a streamlined and efficient approach, Mira provides a robust solution to the vulnerabilities associated with password management in the digital era. 
                                                                                     
## Objectives

- **Offline Security**: Mira prioritizes offline functionality to ensure password management without dependence on internet connectivity. This approach enhances security by minimizing the attack surface and reducing exposure to online threats.                     
- **Local Storage**: The password manager facilitates secure storage and retrieval of passwords locally on the user's machine, ensuring data confidentiality and reducing reliance on external servers or cloud services.

- **No Cloud Dependencies**: Mira does not rely on external servers or cloud services for enhanced privacy and control. This approach eliminates the risk of unauthorized access or data breaches associated with online storage.

- **Data Confidentiality**: To ensure the privacy of user data, Mira implements encryption and secure storage procedures, employing Argon2 for hashing the user data. This ensures that even if the device is compromised, the stored passwords remain unreadable without the decryption key.

- **Isolated Environment**: Mira operates within the user's system, minimizing exposure to external threats or vulnerabilities. This isolated environment enhances security by reducing the likelihood of unauthorized access or data breaches.

- **Usability**: Mira provides a user-friendly command-line interface for easy interaction, allowing users to efficiently manage their passwords without the need for complex technical knowledge.

- **Security Best Practices**: The password manager encourages secure practices such as safeguarding master keys, enabling Two-Factor Authentication (2FA), and emphasizing password complexity to enhance overall security.

- **Control**: Mira empowers users with complete control over their password manager, as it operates locally within the system and functions entirely offline. This provides users with a sense of independence and control over their sensitive information.

## Why MIRA is Considered Secure?

- **Limited Attack Surface**: Operating offline reduces the potential attack surface, minimizing vulnerabilities and enhancing overall security.

- **No Cloud Dependency**: Mira does not store user data in the cloud, reducing the risk of unauthorized access or data breaches associated with online storage. Users have direct control over their data, enhancing privacy and security.

- **Lower Exposure to Online Threats**: Due to its offline nature, Mira is less susceptible to phishing attacks or server breaches common with online password managers. Offline managers are not reliant on online authentication, reducing exposure to online threats.

- **Local Encryption**: Mira employs strong encryption algorithms to protect user data. Data is stored locally, and encryption ensures that even if the device is compromised, the stored passwords remain encrypted and unreadable without the decryption key.

- **Zero-Knowledge Architecture**: Mira ensures the privacy of user data by handling passwords without actually knowing what they are. It encrypts and decrypts data locally, ensuring sensitive information stays private, even from the password manager itself.

- **User Independence**: Users have full control over their password manager since it operates within their system. There's no reliance on external services, providing a sense of independence and reducing potential points of failure.

- **No Password Retrieval Over Network**: Password retrieval and storage occur locally, eliminating the need to transmit sensitive information over the network. This reduces the chances of interception or man-in-the-middle attacks.

- **Offline Password Generation**: Mira can generate passwords without requiring an internet connection, offering convenience and security, especially in situations where online access is limited.

- **No Third-Party Involvement**: Mira operates autonomously without any reliance on external service providers, ensuring enhanced security by eliminating potential risks associated with third-party entities. Users can fully trust the locally implemented security measures without involving external platforms.

- **Increased Privacy Control**: Users benefit from enhanced privacy control as their sensitive data remains within their own environment. There is no need to trust external servers with personal information, contributing to a more private and secure user experience.

- **No Connection Latency**: Users experience minimal latency since their password data is instantly accessible locally. This absence of network communication reduces the risk of delays or disruptions that might be exploited by attackers attempting to compromise the system.

*While offline password managers provide significant security benefits, users should remain vigilant about security best practices. It's crucial to store the master key and decryption key in a secure location on their device. Unfortunately, human error often poses the primary challenge in adhering to these practices.*

## MIRA's Credential Storage Capabilities

MIRA supports various types of credentials, including:

- **Account Passwords**: MIRA enables users to securely store and manage passwords for a variety of accounts, such as those used for websites, applications, and services. It employs encryption techniques to safeguard these passwords, preventing unauthorized access and ensuring user privacy and security online.

- **Credit/Debit Card PINs**: MIRA provides a secure repository for storing sensitive financial information, including PINs for credit and debit cards. By encrypting this data and implementing robust security measures, MIRA ensures that users can safely store and access their card PINs, protecting them from unauthorized use and fraudulent activities. (PREMIUM)

- **API Keys**: MIRA offers a secure environment for storing API keys, which are authentication credentials used to access web services or APIs. By securely managing these keys, MIRA helps prevent unauthorized access to sensitive data and resources, enhancing the overall security of applications and systems that rely on API authentication. (PREMIUM)

- **SSH(RSA) Keys**: MIRA supports the storage and management of SSH keys, which are cryptographic keys used for secure remote access to servers and systems via the Secure Shell (SSH) protocol. By safeguarding SSH keys within its encrypted storage, MIRA helps users securely authenticate and access remote systems, minimizing the risk of unauthorized access and potential security breaches. (PREMIUM)

- **Source Codes**: MIRA allows users to store and protect their project source codes securely. By providing encrypted storage for source code files, MIRA helps developers safeguard their intellectual property and sensitive project information, ensuring that their code remains confidential and protected from unauthorized access or tampering. (PREMIUM)

- **Private Notes**: MIRA enables users to securely store private information, such as personal notes, sensitive documents, or confidential data. By encrypting these notes and providing secure storage, MIRA helps users safeguard their privacy and sensitive information, ensuring that it remains inaccessible to unauthorized individuals or malicious actors. (PREMIUM)

## Features (Few)

- **Comprehensive Credential Storage**: MIRA offers sophisticated storage capabilities, accommodating an extensive range of credentials. These include Platform Accounts, Credit/Debit Card PINs (exclusive to PREMIUM), API Keys (exclusive to PREMIUM), SSH(RSA) Keys (exclusive to PREMIUM), Source Codes (exclusive to PREMIUM), and Private Notes (exclusive to PREMIUM). This breadth of storage options ensures MIRA's flexibility in securely managing diverse types of credentials for users.

- **Multi-Factor Authentication**: Before the user can go to the main menu of MIRA, the user needs to login first. In login prompt, the user needs to enter the master password and the encryption key that has been generated after successful registration and a time-based 6-digit from 2FA if enabled. (2FA is available in PREMIUM)

- **Secure Storage**: Passwords are encrypted using the Fernet symmetric encryption algorithm, ensuring data confidentiality and integrity. Without the encryption key, it will be almost impossible to retrieve or recover the content.

- **Carrier Lookup**: When you use a phone number for your platform credential instead of email, MIRA will detect its Carrier. This feature can help to validate the phone number that has been entered by the user.

- **Mnemonic Phrase**: Since the encryption key is generated in Base64 format, it's hard to memorize and maintain, that's why MIRA can turn the user's encryption key to a memorable and maintainable mnemonic phrase. Just be careful with hypens and underscores and replace them with a letter, number, or symbols that the user will remember and where they are placed before the user convert it to a mnemonic phrase. (PREMIUM)

- **Password Strength Checker**: Ensures that passwords meet recommended strength criteria based on established password policies. (PREMIUM)

- **Password/PINs Expiry Notifications**: Users receive warnings when passwords are about to expire, prompting them to change passwords regularly for enhanced security. (PREMIUM)

- **Password Generator**: Mira offers various password generation methods to create strong and unique passwords tailored to user preferences. (PREMIUM)

- **Password/PINs Expiry Lister**: Allows users to view the remaining time until a password and PIN expires, facilitating proactive password/PIN management. (PREMIUM)

- **Accessible Instructions**: GiraSec prioritized creating intuitive guidance for MIRA, a Terminal-Based Password Manager, ensuring it offers the utmost user-friendliness.

- **Reset Functionality**: In the Premium Version, MIRA offers a reset option designed to provide users with a comprehensive tool for data management. By selecting this option, users can initiate a process that deletes all saved credentials, including their primary MIRA account. This feature serves as a safeguard, enabling users to reset their MIRA instance in situations such as compromised accounts or a need for a fresh start. (PREMIUM)

*Please note: Resetting MIRA will permanently delete all stored credentials, and this action cannot be undone. Exercise caution and ensure that you have backed up any essential data before proceeding with the reset. MIRA will not be able to recover any deleted credentials after the reset is complete.*

## Installation (PIP)

**UNIX-Based**
```bash
sudo pip3 install MIRA-passwdmng
```
**Windows**:
- Run CMD as Administrator  
```bash
pip3 install MIRA-passwdmng
```

## Usage

To use Mira effectively:

- Run the program with elevated privileges using `sudo mira`.
- Typing `h` or `help` in the Mira prompt provides guidance.
- Before logging in, create a master user to secure the password vault.

Note: For **Windows** users, run the **Command Prompt(CMD)** as **Administrator** to run the program.
  

## Commands

Once logged in, users can utilize various commands, including (FOR LITE ONLY):

| Command               | Description                                                      |
|-----------------------|------------------------------------------------------------------|
| add_platform_passwd   | Add a new password for the desired account ID                    |
| get_platform_passwd   | Retrieve the plaintext version of the password for the account ID |
| del_platform_passwd   | Delete a saved password for the desired account ID               |
| ch_platform_pass      | Change the password for the desired account ID                   |
| list_passwd           | List all the stored credentials on your vault                    |
| lout                  | Logout                                                           |
| exit                  | Terminate MIRA                                                   |

## Security Recommendations

To enhance security:

- Keep master and encryption keys secure to prevent unauthorized access.
- Use complex passwords to safeguard against brute-force attacks and unauthorized access attempts.

## Go Premium for Lifetime Access

Unlock a world of possibilities with our Premium membership:
- **Debit/Credit Card PINs are supported**
- **API Keys are supported**
- **SSH(RSA) Keys are supported**
- **Source Codes are supported**
- **Private Notes are supported**
- **No storage limit for Passwords/PINs/SSH(RSA) Keys/API Keys/Source Codes/Private Notes**
- **2-Factor Authentication**
- **User can change the Master Password anytime**
- **Password Generator**
- **Password Strength Checker**
- **Password/PIN Expiry Checker**
- **Password/PIN Expiry Alerts**
- **Account Lockout**
- **Mnemonic Option for encryption key**
- **Reset Option**

*Upgrade today for lifetime access and experience the robustness of GiraSec's MIRA for only $4.50 (250 PHP)!*

## License

- MIRA is licensed under **EULA**.

## For Premium Access, Concerns, and Issues please email us here:

- girasesolutions@gmail.com
- fredmarkivand@gmail.com
- johnrussel205@gmail.com

## Acknowledgements

- GiraSec Solutions
- veilwr4ith
- icode3rror
